package javazoom.Util;
/**
 * This class implements Config feature.
 *
 * @author	E.B from JavaZOOM
 *
 * Homepage : http://www.javazoom.net
 *
 *-----------------------------------------------------------------------
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License as published
 *   by the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *----------------------------------------------------------------------
 */
import java.io.*;
import java.util.*;
import javazoom.Util.ini.*;

/**
 * This class provides all parameters for jlGui coming from a file.
 */
public class Config
{
  private static String	CONFIG_FILE_NAME = "jlgui.ini";
  private Configuration	_config = null;
  // configuration keys
  private static final String
    LAST_URL  = "last_url",
    LAST_DIR  = "last_dir",
    ORIGINE_X = "origine_x",
    ORIGINE_Y = "origine_y",
    LAST_SKIN = "last_skin",
    EXTENSIONS = "allowed_extensions",
    PLAYLIST_IMPL = "playlist_impl",
    LAST_PLAYLIST = "last_playlist",
    PROXY_SERVER = "proxy_server",
    PROXY_PORT = "proxy_port",
    PROXY_LOGIN = "proxy_login",
    PROXY_PASSWORD = "proxy_password",
    PLAYLIST_ENABLED = "playlist_enabled",
    SHUFFLE_ENABLED = "shuffle_enabled",
    REPEAT_ENABLED = "repeat_enabled",
    EQUALIZER_ENABLED = "equalizer_enabled";
  private static Config	_instance = null;
  private String		_extensions = "m3u,wsz,snd,aifc,aif,wav,au,mp1,mp2,mp3,ogg";
  private String		_lastUrl = "http://206.98.167.99:80/";
  private String		_lastDir = "";
  private String		_defaultSkin = "";
  private String		_playlist = "javazoom.jlGui.playlist.BasePlaylist";
  private String		_playlistFilename = "default.m3u";
  private int			_x = 10;
  private int			_y = 10;
  private String		_proxyServer = "";
  private String		_proxyLogin = "";
  private String		_proxyPassword = "";
  private int			_proxyPort = -1;
  private boolean		_playlistEnabled = false;
  private boolean		_shuffleEnabled = false;
  private boolean		_repeatEnabled = false;
  private boolean		_equalizerEnabled = false;


  private Config()
  {}

  /**
   * Returns Config instance.
   */
  public synchronized static Config getInstance()
  {
	  if (_instance == null)
	  {
		_instance = new Config();
	  }
	  return _instance;
  }

  /**
   * Returns playlist filename.
   */
  public String getPlaylistFilename()
  {
	  return _playlistFilename;
  }

  /**
   * Sets playlist filename.
   */
  public void setPlaylistFilename(String pl)
  {
	  _playlistFilename = pl;
  }

  /**
   * Returns last URL.
   */
  public String getLastURL()
  {
	  return _lastUrl;
  }

  /**
   * Sets last URL.
   */
  public void setLastURL(String url)
  {
	  _lastUrl = url;
  }

  /**
   * Returns last Directory.
   */
  public String getLastDir()
  {
	  return _lastDir;
  }

  /**
   * Sets last Directory.
   */
  public void setLastDir(String dir)
  {
	  _lastDir = dir;
  }

  /**
   * Returns audio extensions.
   */
  public String getExtensions()
  {
	  return _extensions;
  }

  /**
   * Returns proxy server.
   */
  public String getProxyServer()
  {
	  return _proxyServer;
  }

  /**
   * Returns proxy port.
   */
  public int getProxyPort()
  {
	  return _proxyPort;
  }

  /**
   * Returns X location.
   */
  public int getXLocation()
  {
	  return _x;
  }

  /**
   * Returns Y location.
   */
  public int getYLocation()
  {
	  return _y;
  }

  /**
   * Sets X,Y location.
   */
  public void setLocation(int x, int y)
  {
	  _x = x;
	  _y = y;
  }

  /**
   * Sets Proxy info.
   */
  public void setProxy(String url, int port, String login, String password)
  {
	  _proxyServer = url;
	  _proxyPort = port;
	  _proxyLogin = login;
	  _proxyPassword = password;
  }


  /**
   * Enables Proxy.
   */
  public boolean enableProxy()
  {
		if ((_proxyServer != null) && (!_proxyServer.equals("")))
		{
			System.getProperties().put( "proxySet", "true" );
            System.getProperties().put( "proxyHost", _proxyServer );
            System.getProperties().put( "proxyPort", ""+_proxyPort );
            return true;
		}
		else return false;
  }

  /**
   * Returns PlaylistUI state.
   */
  public boolean isPlaylistEnabled()
  {
	  return _playlistEnabled;
  }

  /**
   * Sets PlaylistUI state.
   */
  public void setPlaylistEnabled(boolean ena)
  {
	  _playlistEnabled = ena;
  }

  /**
   * Returns ShuffleUI state.
   */
  public boolean isShuffleEnabled()
  {
	  return _shuffleEnabled;
  }

  /**
   * Sets ShuffleUI state.
   */
  public void setShuffleEnabled(boolean ena)
  {
	  _shuffleEnabled = ena;
  }

  /**
   * Returns RepeatUI state.
   */
  public boolean isRepeatEnabled()
  {
	  return _repeatEnabled;
  }

  /**
   * Sets RepeatUI state.
   */
  public void setRepeatEnabled(boolean ena)
  {
	  _repeatEnabled = ena;
  }

  /**
   * Returns EqualizerUI state.
   */
  public boolean isEqualizerEnabled()
  {
	  return _equalizerEnabled;
  }

  /**
   * Sets EqualizerUI state.
   */
  public void setEqualizerEnabled(boolean ena)
  {
	  _equalizerEnabled = ena;
  }

  /**
   * Returns default skin.
   */
  public String getDefaultSkin()
  {
	  return _defaultSkin;
  }

  /**
   * Sets default skin.
   */
  public void setDefaultSkin(String skin)
  {
	  _defaultSkin = skin;
  }

  /**
   * Returns default skin.
   */
  public String getPlaylistClassName()
  {
	  return _playlist;
  }

  /**
   * Loads configuration for the specified file.
   */
  public void load(String configfile)
  {
	CONFIG_FILE_NAME = configfile;
	load();
  }

  /**
   * Loads configuration.
   */
  public void load()
  {
    _config = new Configuration( CONFIG_FILE_NAME );
    if(_config == null) return;

    // Creates config entries if needed.
    if( _config.get(LAST_URL) == null )
      _config.add( LAST_URL, _lastUrl );

    if( _config.get(LAST_DIR) == null )
      _config.add( LAST_DIR, _lastDir );

    if( _config.getInt( ORIGINE_X) == -1 )
      _config.add( ORIGINE_X, _x );

    if( _config.getInt( ORIGINE_Y) == -1 )
      _config.add( ORIGINE_Y, _y );

    if( _config.get(LAST_SKIN) == null )
      _config.add( LAST_SKIN, _defaultSkin );

    if( _config.get(LAST_PLAYLIST) == null )
      _config.add( LAST_PLAYLIST, _playlistFilename );

    if( _config.get(PLAYLIST_IMPL) == null )
      _config.add( PLAYLIST_IMPL, _playlist );

    if( _config.get(EXTENSIONS) == null )
      _config.add( EXTENSIONS, _extensions );

    if( _config.get(PROXY_SERVER) == null )
      _config.add( PROXY_SERVER, _proxyServer );

    if( _config.getInt(PROXY_PORT) == -1 )
      _config.add( PROXY_PORT, _proxyPort );

    if( _config.get(PROXY_LOGIN) == null )
      _config.add( PROXY_LOGIN, _proxyLogin );

    if( _config.get(PROXY_PASSWORD) == null )
      _config.add( PROXY_PASSWORD, _proxyPassword );

    if( _config.getBoolean(PLAYLIST_ENABLED) == false )
      _config.add( PLAYLIST_ENABLED, _playlistEnabled );

    if( _config.getBoolean(SHUFFLE_ENABLED) == false )
      _config.add( SHUFFLE_ENABLED, _shuffleEnabled );

    if( _config.getBoolean(REPEAT_ENABLED) == false )
      _config.add( REPEAT_ENABLED, _repeatEnabled );

    if( _config.getBoolean(EQUALIZER_ENABLED) == false )
      _config.add( EQUALIZER_ENABLED, _equalizerEnabled );

    // Reads config entries
    _lastUrl = _config.get( LAST_URL, _lastUrl );
    _lastDir = _config.get( LAST_DIR, _lastDir );
    _x = _config.getInt( ORIGINE_X, _x );
    _y = _config.getInt( ORIGINE_Y, _y );
    _defaultSkin = _config.get( LAST_SKIN, _defaultSkin );
    _playlistFilename = _config.get( LAST_PLAYLIST, _playlistFilename );
    _extensions = _config.get( EXTENSIONS, _extensions );
    _playlist = _config.get( PLAYLIST_IMPL, _playlist );
    _proxyServer = _config.get( PROXY_SERVER, _proxyServer );
    _proxyPort = _config.getInt( PROXY_PORT, _proxyPort );
    _proxyLogin = _config.get( PROXY_LOGIN, _proxyLogin );
    _proxyPassword = _config.get( PROXY_PASSWORD, _proxyPassword );
    _playlistEnabled = _config.getBoolean( PLAYLIST_ENABLED, _playlistEnabled );
    _shuffleEnabled = _config.getBoolean( SHUFFLE_ENABLED, _shuffleEnabled );
    _repeatEnabled = _config.getBoolean( REPEAT_ENABLED, _repeatEnabled );
    _equalizerEnabled = _config.getBoolean( EQUALIZER_ENABLED, _equalizerEnabled );
  }


  /**
   * Saves configuration.
   */
  public void save()
  {
  	if(_config != null)
  	{
    	_config.add( ORIGINE_X, _x );
    	_config.add( ORIGINE_Y, _y );
    	if (_lastDir != null) _config.add( LAST_DIR, _lastDir );
    	if (_lastUrl != null) _config.add( LAST_URL, _lastUrl );
    	if (_playlistFilename != null) _config.add( LAST_PLAYLIST, _playlistFilename );
    	if (_defaultSkin != null) _config.add( LAST_SKIN, _defaultSkin );
    	_config.add( PLAYLIST_ENABLED, _playlistEnabled );
    	_config.add( SHUFFLE_ENABLED, _shuffleEnabled );
    	_config.add( REPEAT_ENABLED, _repeatEnabled );
    	_config.add( EQUALIZER_ENABLED, _equalizerEnabled );
		_config.save();
	}
  }

}
